@@ -1,3 +1,21 @@
+/* Copyright 2007 Matthew Barnes
+ *
+ * This file is part of GNOME Video Arcade.
+ *
+ * GNOME Video Arcade is free software; you can redistribute it
+ * and/or modify it under the terms of the GNU General Public License
+ * as published by the Free Software Foundation; either version 3 of
+ * the License, or (at your option) any later version.
+ *
+ * GNOME Video Arcade is distributed in the hope that it will be
+ * useful, but WITHOUT ANY WARRANTY; without even the implied warranty
+ * of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+ * GNU General Public License for more details.
+ *
+ * You should have received a copy of the GNU General Public License
+ * along with this program.  If not, see <http://www.gnu.org/licenses/>.
+ */
+
 #include "gva-play-back.h"
 
 #include <errno.h>
@@ -6,11 +24,12 @@
 #include <time.h>
 #include <sys/stat.h>
 
-#include "gva-game-db.h"
+#include "gva-error.h"
 #include "gva-game-store.h"
+#include "gva-mame.h"
 #include "gva-time.h"
+#include "gva-tree-view.h"
 #include "gva-ui.h"
-#include "gva-xmame.h"
 
 static void
 play_back_tree_view_row_activated_cb (GtkTreeView *view,
@@ -63,15 +82,15 @@ play_back_delete_inpfile (GtkTreeRowReference *reference)
         GtkTreeModel *model;
         GtkTreePath *path;
         GtkTreeIter iter;
-        gboolean iter_set;
+        gboolean valid;
         gchar *inpfile;
 
         model = gtk_tree_row_reference_get_model (reference);
 
         path = gtk_tree_row_reference_get_path (reference);
-        iter_set = gtk_tree_model_get_iter (model, &iter, path);
+        valid = gtk_tree_model_get_iter (model, &iter, path);
         gtk_tree_path_free (path);
-        g_assert (iter_set);
+        g_assert (valid);
 
         gtk_tree_model_get (
                 model, &iter, GVA_GAME_STORE_COLUMN_INPFILE, &inpfile, -1);
@@ -124,8 +143,10 @@ play_back_selection_changed_cb (GtkTreeSelection *tree_selection)
 }
 
 static void
-play_back_render_time (GtkTreeViewColumn *column, GtkCellRenderer *renderer,
-                       GtkTreeModel *model, GtkTreeIter *iter)
+play_back_render_time (GtkTreeViewColumn *column,
+                       GtkCellRenderer *renderer,
+                       GtkTreeModel *model,
+                       GtkTreeIter *iter)
 {
         GValue value;
         gchar text[256];
@@ -141,11 +162,14 @@ play_back_render_time (GtkTreeViewColumn *column, GtkCellRenderer *renderer,
 }
 
 static void
-play_back_add_input_file (gchar *inpfile, gchar *romname, GtkTreeModel *model)
+play_back_add_input_file (const gchar *inpfile,
+                          const gchar *name,
+                          GvaGameStore *game_store)
 {
         GtkTreePath *path;
         GtkTreeIter iter;
-        gboolean iter_set;
+        gboolean valid;
+        gchar *inpname;
         gchar *title;
         struct stat statbuf;
         time_t *time;
@@ -158,61 +182,40 @@ play_back_add_input_file (gchar *inpfile, gchar *romname, GtkTreeModel *model)
 
         time = &statbuf.st_ctime;
 
-        path = gva_game_db_lookup (romname);
+        path = gva_tree_view_lookup (name);
         if (path == NULL)
         {
-                g_warning ("%s: Game '%s' not found", inpfile, romname);
+                g_warning ("%s: Game '%s' not found", inpfile, name);
                 return;
         }
 
-        iter_set = gtk_tree_model_get_iter (
-                gva_game_db_get_model (), &iter, path);
-        g_assert (iter_set);
+        valid = gtk_tree_model_get_iter (
+                gva_tree_view_get_model (), &iter, path);
+        g_assert (valid);
 
         gtk_tree_path_free (path);
 
         gtk_tree_model_get (
-                gva_game_db_get_model (), &iter,
-                GVA_GAME_STORE_COLUMN_TITLE, &title, -1);
+                gva_tree_view_get_model (), &iter,
+                GVA_GAME_STORE_COLUMN_DESCRIPTION, &title, -1);
 
-        gtk_list_store_append (GTK_LIST_STORE (model), &iter);
+        gtk_list_store_append (GTK_LIST_STORE (game_store), &iter);
 
         gtk_list_store_set (
-                GTK_LIST_STORE (model), &iter,
+                GTK_LIST_STORE (game_store), &iter,
                 GVA_GAME_STORE_COLUMN_INPFILE, inpfile,
-                GVA_GAME_STORE_COLUMN_ROMNAME, romname,
-                GVA_GAME_STORE_COLUMN_TITLE, title,
+                GVA_GAME_STORE_COLUMN_NAME, name,
+                GVA_GAME_STORE_COLUMN_DESCRIPTION, title,
                 GVA_GAME_STORE_COLUMN_TIME, time,
                 -1);
 
+        inpname = g_strdelimit (g_path_get_basename (inpfile), ".", '\0');
+        gva_game_store_index_insert (game_store, inpname, &iter);
+        g_free (inpname);
+
         g_free (title);
 }
 
-static void
-play_back_refresh_list (GtkWindow *window, GtkTreeView *view)
-{
-        GHashTable *hash_table;
-        GError *error = NULL;
-
-        hash_table = gva_xmame_get_input_files (&error);
-        if (hash_table != NULL)
-        {
-                GtkTreeModel *model;
-
-                model = gtk_tree_view_get_model (view);
-                gtk_list_store_clear (GTK_LIST_STORE (model));
-                g_hash_table_foreach (
-                        hash_table, (GHFunc) play_back_add_input_file, model);
-                g_hash_table_destroy (hash_table);
-        }
-        else
-        {
-                g_assert (error != NULL);
-                g_warning ("%s", error->message);
-                g_clear_error (&error);
-        }
-}
-
 void
 gva_play_back_init (void)
 {
@@ -247,9 +250,9 @@ gva_play_back_init (void)
         g_object_set (renderer, "ellipsize", PANGO_ELLIPSIZE_END, NULL);
         column = gtk_tree_view_column_new_with_attributes (
                 _("Title"), renderer, "text",
-                GVA_GAME_STORE_COLUMN_TITLE, NULL);
+                GVA_GAME_STORE_COLUMN_DESCRIPTION, NULL);
         gtk_tree_view_column_set_sort_column_id (
-                column, GVA_GAME_STORE_COLUMN_TITLE);
+                column, GVA_GAME_STORE_COLUMN_DESCRIPTION);
         gtk_tree_view_append_column (view, column);
 
         g_signal_connect (
@@ -258,9 +261,6 @@ gva_play_back_init (void)
         g_signal_connect (
                 GVA_WIDGET_PLAY_BACK_TREE_VIEW, "row-activated",
                 G_CALLBACK (play_back_tree_view_row_activated_cb), NULL);
-        g_signal_connect (
-                GVA_WIDGET_PLAY_BACK_WINDOW, "show",
-                G_CALLBACK (play_back_refresh_list), view);
         g_signal_connect (
                 GVA_WIDGET_PLAY_BACK_BUTTON, "clicked",
                 G_CALLBACK (play_back_clicked_cb), view);
@@ -276,3 +276,45 @@ gva_play_back_init (void)
 
         play_back_selection_changed_cb (gtk_tree_view_get_selection (view));
 }
+
+void
+gva_play_back_show (const gchar *inpname)
+{
+        GHashTable *hash_table;
+        GvaGameStore *game_store;
+        GtkTreeView *view;
+        GError *error = NULL;
+
+        view = GTK_TREE_VIEW (GVA_WIDGET_PLAY_BACK_TREE_VIEW);
+        game_store = GVA_GAME_STORE (gtk_tree_view_get_model (view));
+
+        hash_table = gva_mame_get_input_files (&error);
+        gva_error_handle (&error);
+
+        if (hash_table != NULL)
+        {
+                gva_game_store_clear (game_store);
+                g_hash_table_foreach (
+                        hash_table, (GHFunc) play_back_add_input_file,
+                        game_store);
+                g_hash_table_destroy (hash_table);
+        }
+
+        if (inpname != NULL)
+        {
+                GtkTreePath *path;
+
+                path = gva_game_store_index_lookup (game_store, inpname);
+
+                if (path != NULL)
+                {
+                        gtk_tree_view_set_cursor (view, path, NULL, FALSE);
+                        gtk_tree_view_scroll_to_cell (
+                                view, path, NULL, TRUE, 0.5, 0.0);
+                        gtk_widget_grab_focus (GTK_WIDGET (view));
+                        gtk_tree_path_free (path);
+                }
+        }
+
+        gtk_widget_show (GVA_WIDGET_PLAY_BACK_WINDOW);
+}
