@@ -1,16 +1,35 @@
+/* Copyright 2007 Matthew Barnes
+ *
+ * This file is part of GNOME Video Arcade.
+ *
+ * GNOME Video Arcade is free software; you can redistribute it
+ * and/or modify it under the terms of the GNU General Public License
+ * as published by the Free Software Foundation; either version 3 of
+ * the License, or (at your option) any later version.
+ *
+ * GNOME Video Arcade is distributed in the hope that it will be
+ * useful, but WITHOUT ANY WARRANTY; without even the implied warranty
+ * of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+ * GNU General Public License for more details.
+ *
+ * You should have received a copy of the GNU General Public License
+ * along with this program.  If not, see <http://www.gnu.org/licenses/>.
+ */
+
 #include "gva-ui.h"
 
 #include <glade/glade.h>
 
-#include "gva-game-db.h"
+#include "gva-error.h"
 #include "gva-favorites.h"
 #include "gva-game-store.h"
 #include "gva-main.h"
+#include "gva-mame.h"
 #include "gva-play-back.h"
 #include "gva-preferences.h"
+#include "gva-process.h"
 #include "gva-tree-view.h"
 #include "gva-util.h"
-#include "gva-xmame.h"
 
 static GladeXML *xml = NULL;
 static GtkUIManager *manager = NULL;
@@ -24,6 +43,29 @@ static const gchar *authors[] =
         NULL
 };
 static const gchar *copyright = "Copyright \xC2\xA9 2007 Matthew Barnes";
+static const gchar *license =
+"GNOME Video Arcade is free software; you can redistribute it "
+"and/or modify it under the terms of the GNU General Public License "
+"as published by the Free Software Foundation; either version 3 of "
+"the License, or (at your option) any later version.\n"
+"\n"
+"GNOME Video Arcade is distributed in the hope that it will be "
+"useful, but WITHOUT ANY WARRANTY; without even the implied warranty "
+"of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the "
+"GNU General Public License for more details.\n"
+"\n"
+"You should have received a copy of the GNU General Public License "
+"along with this program.  If not, see <http://www.gnu.org/licenses/>.";
+
+static void
+record_game_exited (GvaProcess *process, gint status, gchar *inpname)
+{
+        if (process->error == NULL)
+                gva_play_back_show (inpname);
+
+        g_object_unref (process);
+        g_free (inpname);
+}
 
 static void
 action_about_cb (GtkAction *action)
@@ -34,18 +76,16 @@ action_about_cb (GtkAction *action)
         logo = gtk_icon_theme_load_icon (
                 gtk_icon_theme_get_default (),
                 PACKAGE, 128, 0, &error);
-        if (error != NULL)
-        {
-                g_warning ("%s", error->message);
-                g_clear_error (&error);
-        }
+        gva_error_handle (&error);
 
         gtk_show_about_dialog (
                 GTK_WINDOW (GVA_WIDGET_MAIN_WINDOW),
                 "name", PACKAGE_NAME,
                 "version", PACKAGE_VERSION,
-                "comments", _("XMAME Front-End"),
+                "comments", _("M.A.M.E. Front-End"),
                 "copyright", copyright,
+                "license", license,
+                "wrap-license", TRUE,
                 "authors", authors,
                 "translator-credits", _("translator-credits"),
                 "logo", logo,
@@ -84,14 +124,14 @@ action_insert_favorite_cb (GtkAction *action)
         GtkTreeModel *model;
         GtkTreePath *path;
         GtkTreeIter iter;
-        const gchar *romname;
+        const gchar *name;
         gboolean valid;
 
-        romname = gva_tree_view_get_selected_game ();
-        g_assert (romname != NULL);
+        name = gva_tree_view_get_selected_game ();
+        g_assert (name != NULL);
 
-        model = gva_game_db_get_model ();
-        path = gva_game_db_lookup (romname);
+        model = gva_tree_view_get_model ();
+        path = gva_tree_view_lookup (name);
         valid = gtk_tree_model_get_iter (model, &iter, path);
         gtk_tree_path_free (path);
         g_assert (valid);
@@ -100,23 +140,52 @@ action_insert_favorite_cb (GtkAction *action)
                 GTK_LIST_STORE (model), &iter,
                 GVA_GAME_STORE_COLUMN_FAVORITE, TRUE, -1);
 
-        gva_favorites_insert (romname);
+        gva_favorites_insert (name);
 
         gtk_action_set_visible (GVA_ACTION_INSERT_FAVORITE, FALSE);
         gtk_action_set_visible (GVA_ACTION_REMOVE_FAVORITE, TRUE);
 }
 
+static void
+action_next_game_cb (GtkAction *action)
+{
+        GtkTreeSelection *selection;
+        GtkTreeModel *model;
+        GtkTreePath *path;
+        GtkTreeView *view;
+        GtkTreeIter iter;
+        gboolean valid;
+        gint n_nodes;
+        gint index;
+
+        view = GTK_TREE_VIEW (GVA_WIDGET_MAIN_TREE_VIEW);
+        selection = gtk_tree_view_get_selection (view);
+        valid = gtk_tree_selection_get_selected (selection, &model, &iter);
+        n_nodes = gtk_tree_model_iter_n_children (model, NULL);
+        g_return_if_fail (valid);
+
+        path = gtk_tree_model_get_path (model, &iter);
+        index = gtk_tree_path_get_indices (path)[0];
+        index = (index + n_nodes + 1) % n_nodes;
+        gtk_tree_path_free (path);
+
+        path = gtk_tree_path_new_from_indices (index, -1);
+        gtk_tree_view_set_cursor (view, path, NULL, FALSE);
+        gtk_tree_path_free (path);
+}
+
 static void
 action_play_back_cb (GtkAction *action)
 {
+        GvaProcess *process;
         GtkTreeModel *model;
         GtkTreeView *view;
         GtkTreeIter iter;
         gchar *inpfile;
         gchar *inpname;
-        gchar *romname;
+        gchar *name;
         GList *list;
-        gboolean iter_set;
+        gboolean valid;
         GError *error = NULL;
 
         view = GTK_TREE_VIEW (GVA_WIDGET_PLAY_BACK_TREE_VIEW);
@@ -128,23 +197,24 @@ action_play_back_cb (GtkAction *action)
                 gtk_tree_view_get_selection (view), &model);
         g_assert (g_list_length (list) == 1);
 
-        iter_set = gtk_tree_model_get_iter (model, &iter, list->data);
-        g_assert (iter_set);
+        valid = gtk_tree_model_get_iter (model, &iter, list->data);
+        g_assert (valid);
         gtk_tree_model_get (
                 model, &iter, GVA_GAME_STORE_COLUMN_INPFILE, &inpfile,
-                GVA_GAME_STORE_COLUMN_ROMNAME, &romname, -1);
+                GVA_GAME_STORE_COLUMN_NAME, &name, -1);
         inpname = g_strdelimit (g_path_get_basename (inpfile), ".", '\0');
         g_free (inpfile);
 
-        if (!gva_xmame_playback_game (romname, inpname, &error))
-        {
-                g_assert (error != NULL);
-                g_warning ("%s", error->message);
-                g_clear_error (&error);
-        }
+        process = gva_mame_playback_game (name, inpname, &error);
+        gva_error_handle (&error);
+
+        if (process != NULL)
+                g_signal_connect_after (
+                        process, "exited",
+                        G_CALLBACK (g_object_unref), NULL);
 
         g_free (inpname);
-        g_free (romname);
+        g_free (name);
 
         g_list_foreach (list, (GFunc) gtk_tree_path_free, NULL);
         g_list_free (list);
@@ -156,37 +226,67 @@ action_preferences_cb (GtkAction *action)
         gtk_widget_show (GVA_WIDGET_PREFERENCES_WINDOW);
 }
 
+static void
+action_previous_game_cb (GtkAction *action)
+{
+        GtkTreeSelection *selection;
+        GtkTreeModel *model;
+        GtkTreePath *path;
+        GtkTreeView *view;
+        GtkTreeIter iter;
+        gboolean valid;
+        gint n_nodes;
+        gint index;
+
+        view = GTK_TREE_VIEW (GVA_WIDGET_MAIN_TREE_VIEW);
+        selection = gtk_tree_view_get_selection (view);
+        valid = gtk_tree_selection_get_selected (selection, &model, &iter);
+        n_nodes = gtk_tree_model_iter_n_children (model, NULL);
+        g_return_if_fail (valid);
+
+        path = gtk_tree_model_get_path (model, &iter);
+        index = gtk_tree_path_get_indices (path)[0];
+        index = (index + n_nodes - 1) % n_nodes;
+        gtk_tree_path_free (path);
+
+        path = gtk_tree_path_new_from_indices (index, -1);
+        gtk_tree_view_set_cursor (view, path, NULL, FALSE);
+        gtk_tree_path_free (path);
+}
+
 static void
 action_properties_cb (GtkAction *action)
 {
+        gtk_widget_show (GVA_WIDGET_PROPERTIES_WINDOW);
 }
 
 static void
 action_quit_cb (GtkAction *action)
 {
+        gva_process_kill_all ();
         gtk_main_quit ();
 }
 
 static void
 action_record_cb (GtkAction *action)
 {
-        const gchar *romname;
+        GvaProcess *process;
+        const gchar *name;
         gchar *inpname;
         GError *error = NULL;
 
-        romname = gva_tree_view_get_selected_game ();
-        g_assert (romname != NULL);
+        name = gva_tree_view_get_selected_game ();
+        g_assert (name != NULL);
 
-        inpname = gva_choose_inpname (romname);
+        inpname = gva_choose_inpname (name);
 
-        if (!gva_xmame_record_game (romname, inpname, &error))
-        {
-                g_assert (error != NULL);
-                g_warning ("%s", error->message);
-                g_clear_error (&error);
-        }
+        process = gva_mame_record_game (name, inpname, &error);
+        gva_error_handle (&error);
 
-        g_free (inpname);
+        if (process != NULL)
+                g_signal_connect_after (
+                        process, "exited",
+                        G_CALLBACK (record_game_exited), inpname);
 }
 
 static void
@@ -195,14 +295,14 @@ action_remove_favorite_cb (GtkAction *action)
         GtkTreeModel *model;
         GtkTreePath *path;
         GtkTreeIter iter;
-        const gchar *romname;
+        const gchar *name;
         gboolean valid;
 
-        romname = gva_tree_view_get_selected_game ();
-        g_assert (romname != NULL);
+        name = gva_tree_view_get_selected_game ();
+        g_assert (name != NULL);
 
-        model = gva_game_db_get_model ();
-        path = gva_game_db_lookup (romname);
+        model = gva_tree_view_get_model ();
+        path = gva_tree_view_lookup (name);
         valid = gtk_tree_model_get_iter (model, &iter, path);
         gtk_tree_path_free (path);
         g_assert (valid);
@@ -211,49 +311,54 @@ action_remove_favorite_cb (GtkAction *action)
                 GTK_LIST_STORE (model), &iter,
                 GVA_GAME_STORE_COLUMN_FAVORITE, FALSE, -1);
 
-        gva_favorites_remove (romname);
+        gva_favorites_remove (name);
 
         gtk_action_set_visible (GVA_ACTION_INSERT_FAVORITE, TRUE);
         gtk_action_set_visible (GVA_ACTION_REMOVE_FAVORITE, FALSE);
 }
 
+static void
+action_search_cb (GtkAction *action)
+{
+        /* TODO */
+}
+
 static void
 action_show_play_back_cb (GtkAction *action)
 {
-        gtk_widget_show (GVA_WIDGET_PLAY_BACK_WINDOW);
+        gva_play_back_show (NULL);
 }
 
 static void
 action_start_cb (GtkAction *action)
 {
-        const gchar *romname;
+        GvaProcess *process;
+        const gchar *name;
         GError *error = NULL;
 
-        romname = gva_tree_view_get_selected_game ();
-        g_assert (romname != NULL);
+        name = gva_tree_view_get_selected_game ();
+        g_assert (name != NULL);
 
         if (!gva_preferences_get_auto_save ())
-        {
-                if (!gva_xmame_clear_state (romname, &error))
-                {
-                        g_assert (error != NULL);
-                        g_warning ("%s", error->message);
-                        g_clear_error (&error);
-                }
-        }
+                gva_mame_delete_save_state (name);
 
-        if (!gva_xmame_run_game (romname, &error))
-        {
-                g_assert (error != NULL);
-                g_warning ("%s", error->message);
-                g_clear_error (&error);
-        }
+        process = gva_mame_run_game (name, &error);
+        gva_error_handle (&error);
+
+        if (process != NULL)
+                g_signal_connect_after (
+                        process, "exited",
+                        G_CALLBACK (g_object_unref), NULL);
 }
 
 static void
 action_view_changed_cb (GtkRadioAction *action, GtkRadioAction *current)
 {
-        gva_tree_view_update ();
+        GError *error = NULL;
+
+        gva_tree_view_update (&error);
+        gva_error_handle (&error);
+
         gva_tree_view_set_last_selected_view (
                 gva_tree_view_get_selected_view ());
 }
@@ -281,6 +386,13 @@ static GtkActionEntry entries[] =
           N_("Add the selected game to my list of favorites"),
           G_CALLBACK (action_insert_favorite_cb) },
 
+        { "next-game",
+          GTK_STOCK_GO_FORWARD,
+          NULL,
+          "<Alt>rightarrow",
+          N_("Show next game"),
+          G_CALLBACK (action_next_game_cb) },
+
         { "play-back",
           GTK_STOCK_MEDIA_PLAY,
           N_("Play _Back..."),
@@ -295,6 +407,13 @@ static GtkActionEntry entries[] =
           N_("Configure the application"),
           G_CALLBACK (action_preferences_cb) },
 
+        { "previous-game",
+          GTK_STOCK_GO_BACK,
+          NULL,
+          "<Alt>leftarrow",
+          N_("Show previous game"),
+          G_CALLBACK (action_previous_game_cb) },
+
         { "properties",
           GTK_STOCK_PROPERTIES,
           N_("_Properties"),
@@ -323,6 +442,13 @@ static GtkActionEntry entries[] =
           N_("Remove the selected game from my list of favorites"),
           G_CALLBACK (action_remove_favorite_cb) },
 
+        { "search",
+          GTK_STOCK_FIND,
+          N_("S_earch..."),
+          NULL,
+          N_("Show a custom list of games"),
+          G_CALLBACK (action_search_cb) },
+
         { "show-play-back",
           GTK_STOCK_MEDIA_PLAY,
           N_("Play _Back..."),
